//使用删除地图车辆的方式
var useDeleteVehicle = true;
/**
 * 判断给定的对象是否为数组
 * @param {Object} source 要测试的对象
 *
 * @return {Boolean} 如果是数组返回true，否则返回false
 */
function isArray(source) {
    return '[object Array]' === Object.prototype.toString.call(source);
};

/**
 * 返回item在source中的索引位置
 * @param {Object} item 要测试的对象
 * @param {Array} source 数组
 *
 * @return {Number} 如果在数组内，返回索引，否则返回-1
 */
function indexOf(item, source){
    var index = -1;
    if(isArray(source)){
        if (source.indexOf) {
            index = source.indexOf(item);
        } else {
            for (var i = 0, m; m = source[i]; i++) {
                if (m === item) {
                    index = i;
                    break;
                }
            }
        }
    }        
    return index;
};

/**
 * 删除车辆标记信息
 * @param {Object} vehicle 要删除的车辆对象
 */
function deleteVehicleMarkerParam(vehicle) {
	if (vehicle.nameMarker != null){
		GFRAME.map.remove([vehicle.nameMarker.marker_]);
		if(vehicle.nameMarker.marker_){//异常标记点
			GFRAME.gaodeMarkerClusterer.removeMarker(vehicle.nameMarker.marker_);
		}
		vehicle.nameMarker = null;
	}
	if (vehicle.popMarker != null){
		vehicle.popMarker.infowindow.close();
		vehicle.popMarker = null;
	}
	if(typeof deleteVehiTrack == 'function') {
		deleteVehiTrack(vehicle);
	}
	vehicle = null;
}

/**
 * MarkerClusterer
 * @class 用来解决加载大量点要素到地图上产生覆盖现象的问题，并提高性能
 * @constructor
 * @param {Map} map 地图的一个实例。
 * @param {Json Object} options 可选参数，可选项包括：<br />
 *    markers {Array<Marker>} 要聚合的标记数组<br />
 *    girdSize {Number} 聚合计算时网格的像素大小，默认60<br />
 *    maxZoom {Number} 最大的聚合级别，大于该级别就不进行相应的聚合<br />
 *    minClusterSize {Number} 最小的聚合数量，小于该数量的不能成为一个聚合，默认为2<br />
 *    isAverangeCenter {Boolean} 聚合点的落脚位置是否是所有聚合在内点的平均值，默认为否，落脚在聚合内的第一个点<br />
 *    styles {Array<IconStyle>} 自定义聚合后的图标风格，请参考TextIconOverlay类<br />
 */
function MarkerClusterer(map, options) {
	if (!map){
        return;
    }
    this._map = map;
	this._vehiIdnos = [];
    this._clusters = [];
    var opts = options || {};
    this._gridSize = Number(opts["gridSize"]) || 60;
    this._maxZoom =  Number(opts["maxZoom"]) || 18;
    this._minClusterSize =  Number(opts["minClusterSize"]) || 2;
    
	GFRAME.gaodeMarkerClusterer.setMinClusterSize(this._minClusterSize);
	GFRAME.gaodeMarkerClusterer.setMaxZoom(this._maxZoom);
    
    this._isAverageCenter = false;
    if (opts['isAverageCenter'] != undefined) {
        this._isAverageCenter = opts['isAverageCenter'];
    }    
    this._styles = opts["styles"] || [];
    //事件
    this.zoomendEventListener = null;
    this.moveendEventListener = null;
    //初始化点聚合事件
    this.initEventListener();
//    var vids = opts["vehiIdnos"];
//    isArray(vids) && this.addMarkers(vids);
}

/**
 * 初始化点聚合事件
 * @return 无返回值。
 */
MarkerClusterer.prototype.initEventListener = function(){
	var that = this;
	this.zoomendEventListener = function() {
		 that._redraw();
	};
	this.moveendEventListener = function() {
		 that._redraw();
	};
    this._map.on("zoomend", this.zoomendEventListener);
    this._map.on("moveend", this.moveendEventListener);
};

/**
 * 去除点聚合事件
 * @return 无返回值。
 */
MarkerClusterer.prototype.removeEventListener = function(){
	if(this.zoomendEventListener != null) {
		this._map.off('zoomend', this.zoomendEventListener);
	}
	if(this.moveendEventListener != null) {
		this._map.off('moveend', this.moveendEventListener);
	}
};

/**
 * 把一个标记添加到要聚合的标记数组中
 * @param {vehiIdno} vehiIdno 要添加的标记
 *
 * @return 无返回值。
 */
MarkerClusterer.prototype._pushMarkerTo = function(vehiIdno){
	 var index = indexOf(vehiIdno, this._vehiIdnos);
     if(index === -1){
    	 this._vehiIdnos.push(vehiIdno);//Marker拖放后enableDragging不做变化，忽略
     }
};

/**
 * 添加一个聚合的标记。
 * @param {vehiIdno} vehiIdno 要聚合的单个标记。
 * @return 无返回值。
 */
MarkerClusterer.prototype.addMarker = function(vehiIdno) {
		GFRAME.gaodeMarkerClusterer.removeMarker(vehiIdno);//移除
	  	this._pushMarkerTo(vehiIdno);
		var vehicle = GFRAME.vehicleList.get(vehiIdno);
		if(vehicle){
			if(vehicle.nameMarker && vehicle.nameMarker.marker_){
				GFRAME.gaodeMarkerClusterer.addMarker(vehicle.nameMarker.marker_);
			}
		}
};

/**
 * 删除单个标记
 * @param {vehiIdno} vehiIdno 需要被删除的vehiIdno
 *
 * @return {Boolean} 删除成功返回true，否则返回false
 */
MarkerClusterer.prototype.removeMarker = function(vehiIdno) {
    var index = indexOf(vehiIdno, this._vehiIdnos);
    if (index === -1) {
        return false;
    }
    //删除聚合中的标记
    var vehicle = GFRAME.vehicleList.get(vehiIdno);
    if(vehicle){
    	if(vehicle.nameMarker.marker_){
    		GFRAME.gaodeMarkerClusterer.removeMarker(vehicle.nameMarker.marker_);
    	}
//    	deleteVehicleMarkerParam(vehicle);
    }
	this._vehiIdnos.splice(index, 1);
    return true;
};
    
/**
 * 删除一组标记
 * @param {Array<vehiIdno>} vehiIdnos 需要被删除的vehiIdno数组
 *
 * @return {Boolean} 删除成功返回true，否则返回false
 */
MarkerClusterer.prototype.removeMarkers = function(vehiIdnos) {
    var success = false;
    for (var i = 0; i < vehiIdnos.length; i++) {
        var r = this.removeMarker(vehiIdnos[i]);
        success = success || r; 
    }
    return success;
};

/**
 * 从地图上彻底清除所有的标记
 * @return 无返回值
 */
MarkerClusterer.prototype.clearMarkers = function() {
	GFRAME.gaodeMarkerClusterer.clearMarkers();
    this._vehiIdnos = [];
};

/**
 * 重新生成，比如改变了属性等
 * @return 无返回值
 */
MarkerClusterer.prototype._redraw = function () {
	//清除所有
	GFRAME.gaodeMarkerClusterer.clearMarkers();
	var mks = [];
    GFRAME.vehicleList.each(function(vehiIdno, findVehi) {
		if (findVehi != null){
			//当前车辆上一个标记点
			if(findVehi.nameMarker && findVehi.nameMarker.marker_){
				mks.push(findVehi.nameMarker.marker_);
			}
		}
	});
    GFRAME.gaodeMarkerClusterer.addMarkers(mks);
};

/**
 * 设置聚合的最大缩放级别和单个聚合的最小数量
 * @param {Number} maxZoom 聚合的最大缩放级别
 * @param {Number} size 单个聚合的最小数量
 * @return 返回是否改变了
 */
MarkerClusterer.prototype.setClusterZoomAndSize = function(maxZoom, size) {
	if(this._maxZoom != maxZoom || this._minClusterSize != size) {
		this._maxZoom = Number(maxZoom);
		this._minClusterSize = Number(size);
		GFRAME.gaodeMarkerClusterer.setMinClusterSize(this._minClusterSize);
		GFRAME.gaodeMarkerClusterer.setMaxZoom(this._maxZoom);
	    this._redraw();
	}
};